Imports System
Imports System.Collections
Imports System.Reflection
Imports System.Runtime.InteropServices

Imports SldWorks
Imports SWPublished
Imports SwConst
Imports SolidWorksTools

<Guid("7aad1b32-87ce-4db5-ab11-99aad05b8f58")> _
<ComVisible(True)> _
<SwAddin( _
    Description:="$safeprojectname$ description", _
    Title:="$safeprojectname$", _
    LoadAtStartup:=True _
    )> _
    Public Class SwAddin
    Implements SWPublished.SwAddin

#Region "Local Variables"
    Dim WithEvents iSwApp As SldWorks.SldWorks
    Dim iCmdMgr As ICommandManager
    Dim addinID As Integer
    Dim openDocs As Hashtable
    Dim SwEventPtr As SldWorks.SldWorks
    Dim ppage As UserPMPage

    ' Public Properties
    ReadOnly Property SwApp() As SldWorks.SldWorks
        Get
            Return iSwApp
        End Get
    End Property

    ReadOnly Property CmdMgr() As ICommandManager
        Get
            Return iCmdMgr
        End Get
    End Property
#End Region

#Region "SolidWorks Registration"

    <ComRegisterFunction()> Public Shared Sub RegisterFunction(ByVal t As Type)

        ' Get Custom Attribute: SwAddinAttribute
        Dim attributes() As Object
        Dim SWattr As SwAddinAttribute = Nothing

        attributes = System.Attribute.GetCustomAttributes(GetType(SwAddin), GetType(SwAddinAttribute))

        If attributes.Length > 0 Then
            SWattr = DirectCast(attributes(0), SwAddinAttribute)
        End If

        Dim hklm As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.LocalMachine
        Dim hkcu As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.CurrentUser

        Dim keyname As String = "SOFTWARE\SolidWorks\Addins\{" + t.GUID.ToString() + "}"
        Dim addinkey As Microsoft.Win32.RegistryKey = hklm.CreateSubKey(keyname)
        addinkey.SetValue(Nothing, 0)
        addinkey.SetValue("Description", SWattr.Description)
        addinkey.SetValue("Title", SWattr.Title)

        keyname = "Software\SolidWorks\AddInsStartup\{" + t.GUID.ToString() + "}"
        addinkey = hkcu.CreateSubKey(keyname)
        addinkey.SetValue(Nothing, SWattr.LoadAtStartup)
    End Sub

    <ComUnregisterFunction()> Public Shared Sub UnregisterFunction(ByVal t As Type)
        Dim hklm As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.LocalMachine
        Dim hkcu As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.CurrentUser

        Dim keyname As String = "SOFTWARE\SolidWorks\Addins\{" + t.GUID.ToString() + "}"
        hklm.DeleteSubKey(keyname)

        keyname = "Software\SolidWorks\AddInsStartup\{" + t.GUID.ToString() + "}"
        hkcu.DeleteSubKey(keyname)
    End Sub

#End Region

#Region "ISwAddin Implementation"

    Function ConnectToSW(ByVal ThisSW As Object, ByVal Cookie As Integer) As Boolean Implements SWPublished.SwAddin.ConnectToSW
        iSwApp = ThisSW
        addinID = Cookie

        ' Setup callbacks
        iSwApp.SetAddinCallbackInfo(0, Me, addinID)

        ' Setup the Command Manager
        iCmdMgr = iSwApp.GetCommandManager(Cookie)
        AddCommandMgr()

        'Setup the Event Handlers
        SwEventPtr = iSwApp
        openDocs = New Hashtable
        AttachEventHandlers()

        'Setup Sample Property Manager
        AddPMP()

        ConnectToSW = True
    End Function

    Function DisconnectFromSW() As Boolean Implements SWPublished.SwAddin.DisconnectFromSW

        RemoveCommandMgr()
        RemovePMP()
        DetachEventHandlers()

        iSwApp = Nothing
        'The addin _must_ call GC.Collect() here in order to retrieve all managed code pointers 
        GC.Collect()
        DisconnectFromSW = True
    End Function
#End Region

#Region "UI Methods"
    Public Sub AddCommandMgr()

        Dim cmdGroup As ICommandGroup
        Dim iBmp As New BitmapHandler
        Dim thisAssembly As Assembly

        thisAssembly = System.Reflection.Assembly.GetAssembly(Me.GetType())

        cmdGroup = iCmdMgr.CreateCommandGroup(1, "VB Addin", "VB Addin", "", -1)
        cmdGroup.LargeIconList = iBmp.CreateFileFromResourceBitmap("$safeprojectname$.ToolbarLarge.bmp")
        cmdGroup.SmallIconList = iBmp.CreateFileFromResourceBitmap("$safeprojectname$.ToolbarSmall.bmp")
        cmdGroup.LargeMainIcon = iBmp.CreateFileFromResourceBitmap("$safeprojectname$.MainIconLarge.bmp")
        cmdGroup.SmallMainIcon = iBmp.CreateFileFromResourceBitmap("$safeprojectname$.MainIconSmall.bmp")

        cmdGroup.AddCommandItem("CreateCube", -1, "Create a cube", "Create cube", 0, "CreateCube", "", 0)
        cmdGroup.AddCommandItem("Show PMP", -1, "Display sample property manager", "Show PMP", 2, "ShowPMP", "EnablePMP", 2)

        cmdGroup.HasToolbar = True
        cmdGroup.HasMenu = True
        cmdGroup.Activate()

        thisAssembly = Nothing
        iBmp.Dispose()
    End Sub


    Public Sub RemoveCommandMgr()
        iCmdMgr.RemoveCommandGroup(1)
    End Sub


    Function AddPMP() As Boolean
        ppage = New UserPMPage
        ppage.Init(iSwApp, Me)
    End Function

    Function RemovePMP() As Boolean
        ppage = Nothing
    End Function
#End Region

#Region "Event Methods"
    Sub AttachEventHandlers()
        AttachSWEvents()

        'Listen for events on all currently open docs
        Dim modDoc As ModelDoc2
        modDoc = iSwApp.GetFirstDocument()
        While (Not modDoc Is Nothing)
            If (Not openDocs.Contains(modDoc)) Then
                AttachModelDocEventHandler(modDoc)
            End If
            modDoc = modDoc.GetNext
        End While
    End Sub

    Sub DetachEventHandlers()
        DetachSWEvents()

        'Close events on all currently open docs
        Dim docHandler As DocumentEventHandler
        Dim key As ModelDoc2
        Dim numKeys As Integer
        numKeys = openDocs.Count
        If numKeys > 0 Then
            Dim keys() As Object = New Object() {numKeys}

            'Remove all document event handlers
            openDocs.Keys.CopyTo(keys, 0)
            For Each key In keys
                docHandler = openDocs.Item(key)
                docHandler.DetachEventHandlers() 'This also removes the pair from the hash
                docHandler = Nothing
                key = Nothing
            Next
        End If
    End Sub

    Sub AttachSWEvents()
        AddHandler iSwApp.ActiveDocChangeNotify, AddressOf Me.SldWorks_ActiveDocChangeNotify
        AddHandler iSwApp.DocumentLoadNotify, AddressOf Me.SldWorks_DocumentLoadNotify
        AddHandler iSwApp.FileNewNotify2, AddressOf Me.SldWorks_FileNewNotify2
        AddHandler iSwApp.ActiveModelDocChangeNotify, AddressOf Me.SldWorks_ActiveModelDocChangeNotify
    End Sub

    Sub DetachSWEvents()
        RemoveHandler iSwApp.ActiveDocChangeNotify, AddressOf Me.SldWorks_ActiveDocChangeNotify
        RemoveHandler iSwApp.DocumentLoadNotify, AddressOf Me.SldWorks_DocumentLoadNotify
        RemoveHandler iSwApp.FileNewNotify2, AddressOf Me.SldWorks_FileNewNotify2
        RemoveHandler iSwApp.ActiveModelDocChangeNotify, AddressOf Me.SldWorks_ActiveModelDocChangeNotify
    End Sub

    Function AttachModelDocEventHandler(ByVal modDoc As ModelDoc2) As Boolean
        If modDoc Is Nothing Then
            Return False
        End If
        Dim docHandler As DocumentEventHandler = Nothing

        If Not openDocs.Contains(modDoc) Then
            Select Case modDoc.GetType
                Case swDocumentTypes_e.swDocPART
                    docHandler = New PartEventHandler()
                Case swDocumentTypes_e.swDocASSEMBLY
                    docHandler = New AssemblyEventHandler()
                Case swDocumentTypes_e.swDocDRAWING
                    docHandler = New DrawingEventHandler()
            End Select

            docHandler.Init(iSwApp, Me, modDoc)
            docHandler.AttachEventHandlers()
            openDocs.Add(modDoc, docHandler)
        End If
    End Function

    Sub DetachModelEventHandler(ByVal modDoc As ModelDoc2)
        Dim docHandler As DocumentEventHandler
        docHandler = openDocs.Item(modDoc)
        openDocs.Remove(modDoc)
        modDoc = Nothing
        docHandler = Nothing
    End Sub
#End Region

#Region "Event Handlers"
    Function SldWorks_ActiveDocChangeNotify() As Integer
        'TODO: Add your implementation here
    End Function

    Function SldWorks_DocumentLoadNotify(ByVal docTitle As String, ByVal docPath As String) As Integer
        Dim modDoc As ModelDoc2
        modDoc = iSwApp.GetFirstDocument()
        While Not modDoc Is Nothing
            If modDoc.GetTitle = docTitle Then
                If Not openDocs.Contains(modDoc) Then
                    AttachModelDocEventHandler(modDoc)
                End If
            End If
            modDoc = modDoc.GetNext()
        End While
    End Function

    Function SldWorks_FileNewNotify2(ByVal newDoc As Object, ByVal doctype As Integer, ByVal templateName As String) As Integer
        'TODO: Add your implementation here
    End Function

    Function SldWorks_ActiveModelDocChangeNotify() As Integer
        'TODO: Add your implementation here
    End Function
#End Region

#Region "UI Callbacks"
    Sub CreateCube()

        'make sure we have a part open
        Dim partTemplate As String
        Dim model As ModelDoc2
        Dim featMan As FeatureManager

        partTemplate = iSwApp.GetUserPreferenceStringValue(SwConst.swUserPreferenceStringValue_e.swDefaultTemplatePart)
        model = iSwApp.NewDocument(partTemplate, SwConst.swDwgPaperSizes_e.swDwgPaperA2size, 0.0, 0.0)

        model.InsertSketch2(True)
        model.SketchRectangle(0, 0, 0, 0.1, 0.1, 0.1, False)

        'Extrude the sketch
        featMan = model.FeatureManager
        featMan.FeatureExtrusion(True, _
                                  False, False, _
                                  SwConst.swEndConditions_e.swEndCondBlind, SwConst.swEndConditions_e.swEndCondBlind, _
                                  0.1, 0.0, _
                                  False, False, _
                                  False, False, _
                                  0.0, 0.0, _
                                  False, False, _
                                  False, False, _
                                  True, _
                                  False, False)
    End Sub
    Sub ShowPMP()
        ppage.Show()
    End Sub

    Function PMPEnable() As Integer
        If iSwApp.ActiveDoc = Nothing Then
            PMPEnable = 0
        Else
            PMPEnable = 1
        End If
    End Function
#End Region

End Class

#Region "SwAddin class attributes"
<AttributeUsage(AttributeTargets.Class)> _
Class SwAddinAttribute
    Inherits System.Attribute

    Public Description As String = ""
    Public Title As String = ""
    Public LoadAtStartup As Boolean = False

    Sub SwAddinAttribute()
    End Sub

End Class
#End Region

